# NEMO 5 playground

Contains a plyground to run NEMO5 (ver. ) with XIOS TRUNK (ver. 2311).

## Structure

* **nemo5_src** and **xios_2311_src** contain the sources for build the applications.
* The **inputs** are `reference rundirs` with the required namelists and all required files for each resolution.
* **scripts** contains utils to compile and execute, expected to be used as boilerplates.

## Usage

1. Compile XIOS
````bash
# At xios_3211_src dir:
./make_xios --arch mn5_gpp_intel --job 10 --build_dir xios_impi
./make_xios --arch mn5_gpp_intel_openmpi --job 10 --build_dir xios_ompi
````
> If you need to recompile XIOS, use `--full`.

2. Compile NEMO5:

    2.1. Change the ARCH file with the XIOS build dir (nemo5_src/arch/arch-mn5_gpp_intel*)
    ````bash
    %XIOS_INC            -I/your/path/to/xios_2311_src/inc
    %XIOS_LIB            -L/your/path/to/xios_2311_src/lib -lxios -lstdc++
    ````
    2.2. Submit the compilations, several options available
    ````bash
    # At nemo5_src dir:

    source ../scripts/env-intel-mpi.sh      #select the env 

    # INTEL FRO ICE
    ./makenemo -n GLO-INTEL-IMPI-FRO -m mn5_gpp_intel -j 8 -r ORCA2_ICE_PISCES -d "OCE ICE" add_key 'bld::tool::fppkeys key_si3 key_xios key_qco key_vco_1d3d' del_key 'key_top key_RK3'
    
    # INTEL RK3 ICE
    ./makenemo -n GLO-INTEL-IMPI-RK3 -m mn5_gpp_intel -j 8 -r ORCA2_ICE_PISCES -d "OCE ICE" add_key 'bld::tool::fppkeys key_si3 key_xios key_qco key_vco_1d3d key_RK3' del_key 'key_top'
    
    source ../scripts/env-intel-ompi.sh     #select env

    # INTEL+OpenMPI FRO ICE
    ./makenemo -n GLO-INTEL-OMPI-FRO -m mn5_gpp_intel_openmpi -j 8 -r ORCA2_ICE_PISCES -d "OCE ICE" add_key 'bld::tool::fppkeys key_si3 key_xios key_qco key_vco_1d3d' del_key 'key_top key_RK3'

    # INTEL+OpenMPI RK3 ICE
    ./makenemo -n GLO-INTEL-OMPI-RK3 -m mn5_gpp_intel_openmpi -j 8 -r ORCA2_ICE_PISCES -d "OCE ICE" add_key 'bld::tool::fppkeys key_si3 key_xios key_qco key_vco_1d3d key_RK3' del_key 'key_top'


    ````
3. Execute the model:

    3.1. Set the inputs: As the inputs file are very big they cannot be included
    in the repo. For the users under bsc32 you can copy them from:
    `/gpfs/scratch/bsc32/bsc032756/INPUTS/NEMO5/ORCA_025`

    ````bash
    # At inputs/ORCA_025
    cp -r /gpfs/scratch/bsc32/bsc032756/INPUTS/NEMO5/ORCA_025/INPUTS/ .
    ````

    3.2. Check the boilerplate `scripts/run_nemo_orca025.sh` to execute the model and the `--help` from the RUN/run_nemo.sh 
    ````bash
    #!/bin/bash
    ROOT=/gpfs/scratch/bsc32/bsc032756/APPS/NEMO5_INSTALL_TEST
    ROOT_NEMO="$ROOT/nemo5_src"
    ROOT_XIOS="$ROOT/xios_2311_src"
    ROOT_RUNDIR="$ROOT/RUNDIR"

    # RUN INTEL MPI:
    ENV_FILE="env-intel-mpi.sh"

    ./RUN/run_nemo.sh  \
            --nemo "${ROOT_NEMO}/cfgs/GLO-INTEL-IMPI-FRO/EXP00" \
            --xios "${ROOT_XIOS}/xios_impi/bin/xios_server.exe" \
            --env "${ENV_FILE}" \
            --rundir ${ROOT_RUNDIR}/TEST/ORCA025/IMPI \
            --resolution ORCA025-FRO --force

    # Using ORCA025-RK3 uses the same inputs.

    ~ Usage: run_nemo.sh 
    ~ 	--nemo   <NEMO_CFG_PATH>
    ~ 	--xios   <XIOS_BIN>
    ~ 	--env    <ENV_FILE
    ~ 	--rundir <RUNDIR_PATH>
    ~ 	--resolution  ORCA025|ORCA12|ORCA36
    ~ OPTIONAL FLAGS:
    ~ [--nodes=<nodes>] [--MPI=<val>] [--OpenMP=<val>] 
    ~ [--timelimit=<hh:mm:ss>]
    ~ [--account=<SLURM-ACC>] [--queue=<queue>] [--wait] [--contiguous]
    ~ [--force] [--dry] [--help] [--highmem]

    ````
