#!/bin/bash

# Author: Victor Correal (victor.correal@bsc.es)
# Date  : 28/06/2024

# IMPORTS:
SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
source ${SCRIPT_DIR}/bash_utils.sh
source ${SCRIPT_DIR}/run_nemo_utils.sh
source ${SCRIPT_DIR}/run_setups.sh

# SIM VARIABLES:
NEMO=
XIOS_BIN=
ENV_FILE=
RUNDIR=
RESOLUTION=
FORCE_RUN=false
DRY=false

# SLURM VARIABLES:
ACCOUNT=DEFAULT
NODES=DEFAULT
QUEUE=DEFAULT
TIME_LIMIT=DEFAULT
MPI_PER_NODE=DEFAULT
OMP_PER_TASK=DEFAULT
SLURM_HIGHMEM=false
SLURM_WAIT=false
SLURM_CONTIGUOUS=false
SLURM_ARGS=""

# PARSE ARGS && USER CHECK(Y/N):
parse_args "$@"

check_required_args
check_dirs_and_bins
check_slurm_options
print_options
check_user

# COPY BINARIES & ENV:
mkdir -p $RUNDIR

cp -rL $NEMO/nemo $RUNDIR/nemo.exe
cp $XIOS_BIN $RUNDIR/xios_server.exe
ln -sf $PWD/$ENV_FILE $RUNDIR/env.sh

FLS="axis_def_nemo.xml grid_def_nemo.xml domain_def_nemo.xml field_def_nemo-ice.xml field_def_nemo-oce.xml  "
FLS+="namelist_ref namelist_ice_ref namelist_top_ref "

for f in `echo $FLS` ; do
    cp $NEMO/../../SHARED/${f} $RUNDIR
done

# COPY CONFIGURATIONS, RESTARTS & FORCINGS:
info "using $RESOLUTION, loading inputs into rundir"
case "$RESOLUTION" in
    ORCA025-FRO)
        orca025_common
    ;;
    ORCA025-RK3)
        orca025_common
        info "Let rn_Dt as default!"
    ;;
    ORCA025-RK3-2400)
        orca025_common
        orca025_rk3_2400
    ;;
    ORCA025-RK3-3600)
        orca025_common
        orca025_rk3_3600
    ;;
    ORCA12)
        fatal "Not implemented yet"
    ;;
    ORCA36-RK3)
        fatal "Not implemented yet"
    ;;
    ORCA36-FRO)
        fatal "Not implemented yet"
        
    ;;
    *)
        fatal "INVALID RESOLUTION: $RESOLUTION"
    ;;
esac

# COPY SLURM SCRIPT:
info "SETTING UP SLURM SCRIPT..."
#cp ${SCRIPT_DIR}/run_nemo_het.slurm $RUNDIR/run_nemo.slurm
cp ${SCRIPT_DIR}/run_nemo.slurm $RUNDIR/run_nemo.slurm

# SUBMIT IF ! DRY:
if [ $DRY = true ]; then
    success "DRY -> NO SUBMIT!"
else
    cd $RUNDIR
    JOB_ID=$(sbatch $SLURM_ARGS --parsable run_nemo.slurm $PROFILER)
    success "SUBMITTED -> $JOB_ID"
fi


