#!/bin/bash

SCRIPT_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
source ${SCRIPT_DIR}/bash_utils.sh
source ${SCRIPT_DIR}/run_nemo_utils.sh

#===================================ORCA025=====================================
orca025_common(){
    
    ROOT_INPUT=$SCRIPT_DIR/../../inputs/ORCA_025
    BENCH_ROOT=$ROOT_INPUT/BENCH    
    INPUT_ROOT=$ROOT_INPUT/INPUTS

    info "Getting inputs from $BENCH_ROOT and $INPUT_ROOT"

    info "MANAGE NAMELISTS"
    cp $BENCH_ROOT/namelist_cfg_ORCA025_MLF1200 $RUNDIR/namelist_cfg
    cp $BENCH_ROOT/namelist_ice_cfg_ORCA025 $RUNDIR/namelist_ice_cfg

    info "MANAGE I/O SETUP"
    cp $BENCH_ROOT/iodef.xml $RUNDIR
    cp $BENCH_ROOT/context_nemo.xml $RUNDIR

    info "MANAGE CONFIG & FORCINGS"
    ln -sf $INPUT_ROOT/iORCA_R025_*.nc $RUNDIR
    ln -sf $INPUT_ROOT/potential_temperature_WOA13_A5B2_Reg1L75_clim.nc $RUNDIR
    ln -sf $INPUT_ROOT/practical_salinity_WOA13_A5B2_Reg1L75_clim.nc $RUNDIR
    ln -sf $INPUT_ROOT/ERA5_*.nc $RUNDIR
    ln -sf $INPUT_ROOT/iORCA_R025_domain_cfg.nc $RUNDIR/domain_cfg.nc

    info "MANAGE RESTARTS"
    ln -sf $INPUT_ROOT/iORCA_R025_GLO12_ng_1d-m_20230104-20230104_restart_ice.nc $RUNDIR/Ice_initialization.nc
    ln -sf $INPUT_ROOT/iORCA_R025_GLO12_ng_1d-m_20230104-20230104_restart_ice.nc $RUNDIR/restart_ice.nc
    ln -sf $INPUT_ROOT/iORCA_R025_GLO12_ng_1d-m_20230104-20230104_restart_oce.nc $RUNDIR/restart.nc

    info "*** Disabling restarts! (nn_stock=-1)"
    sed -i '58i\    nn_stock = -1' $RUNDIR/namelist_cfg
}

orca025_rk3_2400(){
    change_namelist_parameter $RUNDIR/namelist_cfg ln_rstart .FALSE.
    change_namelist_parameter $RUNDIR/namelist_cfg rn_Dt 2400
    change_namelist_parameter $RUNDIR/namelist_cfg nn_e 120
}

orca025_rk3_3600(){
    change_namelist_parameter $RUNDIR/namelist_cfg ln_rstart .FALSE.
    change_namelist_parameter $RUNDIR/namelist_cfg rn_Dt 3600
    change_namelist_parameter $RUNDIR/namelist_cfg nn_e 180
}

